package com.zsc.edu.gateway.modules.iot.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.product.dto.ProductDto;
import com.zsc.edu.gateway.modules.iot.product.entity.Product;
import com.zsc.edu.gateway.modules.iot.product.entity.ProductParam;
import com.zsc.edu.gateway.modules.iot.product.query.ProductQuery;
import com.zsc.edu.gateway.modules.iot.product.repo.ProductParamRepository;
import com.zsc.edu.gateway.modules.iot.product.repo.ProductRepository;
import com.zsc.edu.gateway.modules.iot.product.service.ProductService;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


/**
 * @author zhaung
 */
@AllArgsConstructor
@Service
public class ProductServiceImpl extends ServiceImpl<ProductRepository, Product> implements ProductService {
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepo;
    private final ProductParamRepository productParamRepo;

    /**
     * 新建产品
     */
    @Override
    public Product create(ProductDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该设备已存在！");
        }
        Product product = new Product();
        product.setName(dto.getName());
        product.setTypeString(dto.getType());
        product.setModel(dto.getModel());
        product.setLink(dto.getLink());
        save(product);
        List<Long> paramIds = new ArrayList<>();
        if (dto.getParams() != null) {
            dto.getParams().forEach(paramDto -> {
                Param param = paramMapper.toEntity(paramDto);
                paramRepo.insert(param);
                paramIds.add(param.getId());
            });
        }
        List<ProductParam> productParams = paramIds.stream()
                .map(paramId -> new ProductParam(product.getId(), paramId))
                .toList();
        productParamRepo.insert(productParams);
        return product;
    }

    /**
     * 更新产品
     */
    @Override
    public Product update(ProductDto dto, Long id) {
        // 更新产品信息
        this.lambdaUpdate().eq(Product::getId, id)
                .set(Product::getName, dto.getName())
                .set(Product::getTypeString, dto.getType())
                .set(Product::getModel, dto.getModel())
                .set(Product::getLink, dto.getLink())
                .update();
        // 通过联表获取paramIds
        LambdaQueryWrapper<ProductParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ProductParam::getProductId, id);
        List<ProductParam> productParams = productParamRepo.selectList(wrapper);
        List<Long> paramIds = productParams.stream()
                .map(ProductParam::getParamId)
                .toList();
        //通过paramIds查询出所有param
        List<Param> params = paramRepo.selectByIds(paramIds);
        //更新param参数
        if (dto.getParams() != null && !dto.getParams().isEmpty()) {
            for (int i = 0; i < dto.getParams().size(); i++) {
                ParamDto paramDto = dto.getParams().get(i);
                Param param = params.get(i);
                paramMapper.convert(paramDto, param);
                paramRepo.updateById(param);
            }
        }
        return this.getById(id);
    }

    /**
     * 分页查询产品
     *
     * @param query 查询表单
     * @param page  分页参数
     */
    @Override
    public IPage<Product> page(Page<Product> page, ProductQuery query) {
        return baseMapper.page(page, query);
    }

    public Product detail(Long id) {
        return baseMapper.selectById(id);
    }

    /**
     * 删除
     */
    @Override
    public boolean delete(Long id) {
        removeById(id);
        LambdaQueryWrapper<ProductParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ProductParam::getProductId, id);
        List<ProductParam> productParams = productParamRepo.selectList(wrapper);
        if (!productParams.isEmpty()) {
            List<Long> paramIds = productParams.stream()
                    .map(ProductParam::getParamId)
                    .toList();
            paramRepo.deleteByIds(paramIds);
        }
        productParamRepo.delete(wrapper);
        return true;
    }
}
